import places_summarized


class NoPlacesError(Exception):
    def __init__(self, location):
        self.location = location

    def __str__(self):
        return "No places found for location {}".format(self.location)



import unittest
from places_summarized.fake_client import FakeClient


class TestSummary(unittest.TestCase):
    def setUp(self):
        self.fclient = FakeClient()
        self.summary = self.fclient.places_summary()

    def test_average_price_level(self):
        r = self.summary.result()
        self.assertEqual(r['average_price_level'], 2.5)

    def test_average_rating(self):
        self.assertAlmostEqual(self.summary.average_rating_by_type('restaurant'), 3.93, 2)


if __name__ == '__main__':
    unittest.main()
    


# Some of the param, type descriptions and error messages are taken from
# the the Google Maps API for Python source code:
# https://github.com/googlemaps/google-maps-services-python/blob/master/googlemaps/places.py
# This was done in good faith to be consistent with the Maps API.

import places_summarized
from places_summarized.errors import NoPlacesError
from statistics import StatisticsError, mean


class Summary(object):
    """
    Performs the summary using Google's nearby places data.
    """

    def __init__(self, result, location):
        self.nearby_results = result
        self.location = location
        self.next_page_token = result['next_page_token']

    def result(self):
        """
        Returns the summary
        """
        return self._summary

    def ratings_by_type(self, location_type):
        """
        Returns the ratings of the locations of the given type

        :param type: Location type, e.g., atm, park, or restaurant.
        You can find the full list at: https://developers.google.com/places/web-service/supported_types
        :type location_type: string
        """
        return self._get_stat_by_type('rating', location_type)

    def average_rating_by_type(self, location_type):
        """
        Calculates the average rating of the locations of the given type

        :param type: Location type, e.g., atm, park, or restaurant.
        You can find the full list at: https://developers.google.com/places/web-service/supported_types
        :type location_type: string
        """
        if self.nearby_results is None:
            raise NoPlacesError(self.location)

        ratings = self.ratings_by_type(location_type)
        try:
            return mean(ratings)
        except StatisticsError:
            print("No ratings found.")

    def average_price_level(self):
        try:
            return mean(self.price_levels)
        except StatisticsError:
            print("No ratings found.")

    def _make_summary(self):
        locations = self.nearby_results.get('results', [])
        num_locations = len(locations)

        if num_locations == 0:
            raise NoPlacesError(self.location)

        self.num_locations = num_locations
        self.ratings = []
        self.user_ratings_totals = []
        self.price_levels = []
        self.location_types = {}

        for location in locations:
            _append_if_key_exists(self.ratings, location, 'rating')
            _append_if_key_exists(self.user_ratings_totals,
                                  location, 'user_ratings_total')
            _append_if_key_exists(self.price_levels, location, 'price_level')

            for location_type in location.get('types', []):
                # Check if location_types has a type of that location.
                # If it does, increase it counter, if it doesnt, a new
                # key is created with value 1.
                self.location_types[location_type] = self.location_types.get(
                    location_type, 0) + 1
        summary = {
            'ratings': self.ratings,
            'user_ratings_total': self.user_ratings_totals,
            'price_levels': self.price_levels,
            'location_types': self.location_types
        }

        _add_average_if_values(self.ratings, summary,
                               'average_rating')
        _add_average_if_values(self.user_ratings_totals, summary,
                               'average_user_ratings_total')
        _add_average_if_values(self.price_levels, summary,
                               'average_price_level')

        self._summary = summary

    def _get_stat_by_type(self, stat, location_type):
        if self.nearby_results is None:
            raise NoPlacesError(self.location)

        values = []
        locations = self.nearby_results.get('results', [])
        for location in locations:
            if location_type in location.get('types', []):
                _append_if_key_exists(values, location, stat)

        return values


def _append_if_key_exists(l, d, key):
    if key in d.keys():
        l.append(d[key])


def _add_average_if_values(l, d, key):
    if l is None or len(l) == 0:
        return

    d[key] = mean(l)



# Some of the param, type descriptions and error messages are taken from
# the the Google Maps API for Python source code:
# https://github.com/googlemaps/google-maps-services-python/blob/master/googlemaps/places.py
# This was done in good faith to be consistent with the Maps API.

import places_summarized
import googlemaps
import json
import os
from places_summarized.summary import Summary


class Client(object):
    """
    Connects to Google Maps, does requests to Google Places API.
    """

    def __init__(self, key=None):
        """
        :param key: Maps API key.
        :type key: string
        """

        self.key = key
        self.gmaps_client = googlemaps.Client(key=key)

    def places_nearby(self, location=None, radius=None, page_token=None):
        """
        Get the places near the location given.

        :param location: The latitude/longitude value for which you wish to
                         obtain the nearby locations.
        :type location: string, dict, list, or tuple

        :param radius: Distance (in meters) from the location.
        :type radius: int

        :param page_token: Token from a previous search that when provided will
        returns the next page of results for the same search.
        """
        if not location and not page_token:
            raise ValueError(
                "either a location or page_token argument is required")

        return self.gmaps_client.places_nearby(location=location,
                                               radius=radius,
                                               page_token=page_token)

    def places_summary(self, location=None, radius=None, page_token=None):
        """
        Get the places near the location given, and compute it summary.

        :param location: The latitude/longitude value for which you wish to
                         obtain a summary of the nearby locations.
        :type location: string, dict, list, or tuple

        :param radius: Distance (in meters) from the location.
        :type radius: int

        :param page_token: Token from a previous search that when provided will
        returns the next page of results for the same search.

        :return: A Summary object.
        :rtype: places_summarized.summary.Summary
        """
        if not location and not page_token:
            raise ValueError(
                "either a location or page_token argument is required")

        result = self.places_nearby(
            location=location, radius=radius, page_token=page_token)

        summary = Summary(result, location)
        summary._make_summary()
        return summary



import json
import os
from places_summarized.client import Client
from places_summarized.summary import Summary


class FakeClient(Client):
    """
    Client used for testing purposes. Loads a local dataset containing
    the locations from a locality from Sydney.
    """

    def __init__(self, key=None):
        pass

    def places_nearby(self, location=None, radius=None, page_token=None):
        print("Method not available in FakeClient.")

    def places_summary(self):
        """
        Computes the summary of a vecinity loaded from a local file.
        Use for testing purposes.
        """

        with open(os.path.join(os.path.dirname(__file__), 'data/sydney.json')) as json_file:
            data = json.load(json_file)

        summary = Summary(data, "Test Location", "Test Page Token")
        summary._make_summary()
        return summary



import os
import setuptools

base_packages = ["googlemaps>=3.1.4"]


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setuptools.setup(
    name="places-summarized",
    version="0.0.9",
    author="Juan De Dios Santos",
    author_email="author@example.com",
    description="Summarizing the Google Maps attributes of places within a specified area",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/juandes/places-summarized",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
    ],
    python_requires=">=3.6",
    install_requires=base_packages,
    include_package_data=True
)
